// src/engine/DataEngine.ts
var DataEngine = class _DataEngine {
  static IsInstance(object) {
    return object instanceof _DataEngine;
  }
};

// src/utils/Catcher.ts
var Catcher = class {
  static async CatchError(promise) {
    return promise.then((v) => [void 0, v]).catch((v) => [v]);
  }
};

// src/utils/Debounce.ts
var Debounce = class {
  delay;
  _executing;
  _timers;
  _promises;
  _pendingExecutions;
  _currentExecutions;
  constructor(delay) {
    this.delay = delay;
    this._executing = /* @__PURE__ */ new Set();
    this._timers = /* @__PURE__ */ new Map();
    this._promises = /* @__PURE__ */ new Map();
    this._pendingExecutions = /* @__PURE__ */ new Map();
    this._currentExecutions = /* @__PURE__ */ new Map();
  }
  /**
   * Executes a function after debouncing any previous calls with the same ID.
   * @param id The identifier for this execution group
   * @param execute The function to execute
   * @returns A promise that resolves with the result of the executed function
   */
  execute(id, execute) {
    this._pendingExecutions.set(id, execute);
    if (this._timers.has(id)) {
      clearTimeout(this._timers.get(id));
    }
    const promise = new Promise((resolve, reject) => {
      if (!this._promises.has(id)) {
        this._promises.set(id, []);
      }
      this._promises.get(id).push({ resolve, reject });
    });
    this._timers.set(id, setTimeout(async () => {
      const fn = this._pendingExecutions.get(id);
      this._timers.delete(id);
      this._pendingExecutions.delete(id);
      this._executing.add(id);
      const executionPromise = fn();
      this._currentExecutions.set(id, executionPromise);
      const [err, result] = await Catcher.CatchError(executionPromise);
      this._executing.delete(id);
      this._currentExecutions.delete(id);
      const promises = this._promises.get(id) || [];
      if (err) {
        promises.forEach((p) => p.reject(err));
        this._promises.delete(id);
      } else {
        promises.forEach((p) => p.resolve(result));
        this._promises.delete(id);
        return result;
      }
    }, this.delay));
    return promise;
  }
  /**
   * Cancels any pending execution with the given ID.
   * @param id The identifier for the execution group to cancel
   */
  cancel(id) {
    const cancellationError = new Error(`Execution with ID ${id} was cancelled`);
    if (this._timers.has(id)) {
      clearTimeout(this._timers.get(id));
      this._timers.delete(id);
    }
    if (this._promises.has(id)) {
      const promises = this._promises.get(id);
      promises.forEach((p) => p.reject(cancellationError));
      this._promises.delete(id);
    }
    this._pendingExecutions.delete(id);
    this._executing.delete(id);
    this._currentExecutions.delete(id);
  }
  /**
   * Checks if an execution with the given ID is currently running.
   * @param id The identifier to check
   * @returns True if the execution is running, false otherwise
   */
  isExecuting(id) {
    return this._executing.has(id);
  }
  /**
   * Checks if an execution with the given ID is currently being debounced.
   * @param id The identifier to check
   * @returns True if the execution is being debounced, false otherwise
   */
  isDebouncing(id) {
    return this._timers.has(id);
  }
  /**
   * Waits for any pending or executing function with the given ID to complete.
   * @param id The identifier to wait for
   * @returns A promise that resolves with the result of the executed function
   */
  async done(id) {
    if (!this.isDebouncing(id) && !this.isExecuting(id)) {
      return void 0;
    }
    if (this.isExecuting(id) && this._currentExecutions.has(id)) {
      return this._currentExecutions.get(id);
    }
    return new Promise((resolve, reject) => {
      if (!this._promises.has(id)) {
        this._promises.set(id, []);
      }
      this._promises.get(id).push({ resolve, reject });
    });
  }
};

// src/engine/WebWorker.ts
var WebWorkerStrategy = class extends DataEngine {
  directoryHandle;
  fileHandle;
  constructor(directoryHandle, fileHandle) {
    super();
    this.directoryHandle = directoryHandle;
    this.fileHandle = fileHandle;
  }
};
var FileSystemSyncAccessHandleStrategy = class extends WebWorkerStrategy {
  accessHandle;
  constructor(directoryHandle, fileHandle) {
    super(directoryHandle, fileHandle);
  }
  async exists(file) {
    if (!this.directoryHandle) {
      return false;
    }
    let exists = false;
    for await (const key of this.directoryHandle.keys()) {
      if (key === file) {
        exists = true;
        break;
      }
    }
    if (exists) {
      exists = !!await this.size();
    }
    return exists;
  }
  async boot(file) {
    this.accessHandle = await this.fileHandle.createSyncAccessHandle();
  }
  async create(file, initialData) {
    this.append(initialData);
  }
  async open(file) {
  }
  async close() {
    this.accessHandle.flush();
    this.accessHandle.close();
  }
  async size() {
    return this.accessHandle.getSize();
  }
  async read(start, length) {
    if (length === void 0) {
      length = await this.size() - start;
    }
    const size = Math.min(await this.size() - start, length);
    const buffer = new DataView(new ArrayBuffer(size));
    const chunk = new Array(buffer.byteLength);
    this.accessHandle.read(buffer, { at: start });
    for (let i = 0; i < size; i++) {
      chunk[i] = buffer.getUint8(i);
    }
    return chunk;
  }
  async update(start, data) {
    const size = await this.size();
    const length = Math.min(data.length, size - start);
    const chunk = data.slice(0, length);
    const buf = Uint8Array.from(chunk);
    this.accessHandle.write(buf, { at: start });
    this.accessHandle.flush();
    return chunk;
  }
  async append(data) {
    const before = await this.size();
    this.accessHandle.truncate(before + data.length);
    this.accessHandle.flush();
    await this.update(before, data);
  }
  async truncate(size) {
    this.accessHandle.truncate(size);
  }
  async unlink(file) {
    await this.close();
    await this.directoryHandle.removeEntry(file);
  }
};
var WritableStreamStrategy = class extends WebWorkerStrategy {
  fileData;
  _debounce;
  constructor(directoryHandle, fileHandle) {
    super(directoryHandle, fileHandle);
    this._debounce = new Debounce(100);
  }
  async _commit() {
    return this._debounce.execute("commit", async () => {
      const stream = await this.fileHandle.createWritable();
      await stream.write(this.fileData);
      await stream.close();
    });
  }
  async exists(file) {
    if (!this.directoryHandle) {
      return false;
    }
    let exists = false;
    for await (const key of this.directoryHandle.keys()) {
      if (key === file) {
        exists = true;
        break;
      }
    }
    if (exists) {
      exists = !!await this.size();
    }
    return exists;
  }
  async boot(file) {
    const rawFile = await this.fileHandle.getFile();
    this.fileData = new Uint8Array(await rawFile.arrayBuffer());
  }
  async create(file, initialData) {
    this.append(initialData);
    this._commit();
  }
  async open(file) {
  }
  async close() {
    await this._commit();
  }
  async size() {
    return this.fileData.length;
  }
  async read(start, length) {
    if (length === void 0) {
      length = await this.size() - start;
    }
    const size = Math.min(await this.size() - start, length);
    return Array.from(this.fileData.slice(start, start + size));
  }
  async update(start, data) {
    const size = await this.size();
    const length = Math.min(data.length, size - start);
    const chunk = data.slice(0, length);
    this.fileData.set(chunk, start);
    this._commit();
    return chunk;
  }
  async append(data) {
    this.fileData = new Uint8Array([...this.fileData, ...data]);
    this._commit();
  }
  async truncate(size) {
    this.fileData = this.fileData.slice(0, size);
    this._commit();
  }
  async unlink(file) {
    await this.close();
    await this.directoryHandle.removeEntry(file);
  }
};
var WebWorkerEngine = class extends DataEngine {
  booting = false;
  strategy;
  constructor() {
    super();
  }
  get fileHandle() {
    return this.strategy?.fileHandle ?? null;
  }
  async _getHandles(file) {
    const directoryHandle = await navigator.storage.getDirectory();
    const fileHandle = await directoryHandle.getFileHandle(file, { create: true });
    return [
      directoryHandle,
      fileHandle
    ];
  }
  async _getBestStrategy(directoryHandle, fileHandle) {
    if (typeof globalThis.window !== "undefined") {
      return new WritableStreamStrategy(directoryHandle, fileHandle);
    } else if ("ServiceWorkerGlobalScope" in self && self instanceof ServiceWorkerGlobalScope) {
      return new WritableStreamStrategy(directoryHandle, fileHandle);
    } else if ("SharedWorkerGlobalScope" in self && self instanceof SharedWorkerGlobalScope) {
      return new WritableStreamStrategy(directoryHandle, fileHandle);
    } else if ("DedicatedWorkerGlobalScope" in self && self instanceof DedicatedWorkerGlobalScope) {
      return new FileSystemSyncAccessHandleStrategy(directoryHandle, fileHandle);
    } else {
      throw new Error("Unknown environment.");
    }
  }
  async exists(file) {
    return this.strategy.exists(file);
  }
  async boot(file) {
    if (this.booting) {
      return;
    }
    this.booting = true;
    const [directoryHandle, fileHandle] = await this._getHandles(file);
    this.strategy = await this._getBestStrategy(directoryHandle, fileHandle);
    return this.strategy.boot(file);
  }
  async create(file, initialData) {
    return this.strategy.create(file, initialData);
  }
  async open(file) {
    return this.strategy.open(file);
  }
  async close() {
    return this.strategy.close();
  }
  async size() {
    return this.strategy.size();
  }
  async read(start, length) {
    return this.strategy.read(start, length);
  }
  async update(start, data) {
    return this.strategy.update(start, data);
  }
  async append(data) {
    return this.strategy.append(data);
  }
  async truncate(size) {
    return this.strategy.truncate(size);
  }
  async unlink(file) {
    await this.close();
    await this.strategy.unlink(file);
  }
};
export {
  WebWorkerEngine
};
