import { Ryoiki } from 'ryoiki';
import { DataJournal, DataJournalContainer } from '../engine/DataJournal';
import { DataEngine } from '../engine/DataEngine';
import { KlafCreateOption } from './Klaf';
export interface KlafMetadata {
    majorVersion: number;
    minorVersion: number;
    patchVersion: number;
    payloadSize: number;
    timestamp: bigint;
    secretKey: bigint;
    autoIncrement: bigint;
    count: number;
    index: number;
    lastInternalIndex: number;
}
export declare enum KlafFormat {
    DB_VERSION = "8.0.0",
    DB_NAME = "TissueRoll",
    MetadataValidStringOffset = 0,
    MetadataValidStringSize,
    MetadataMajorVersionOffset,
    MetadataMajorVersionSize = 1,
    MetadataMinorVersionOffset,
    MetadataMinorVersionSize = 1,
    MetadataPatchVersionOffset,
    MetadataPatchVersionSize = 1,
    MetadataIndexOffset,
    MetadataIndexSize = 4,
    MetadataPayloadSizeOffset,
    MetadataPayloadSizeSize = 4,
    MetadataTimestampOffset,
    MetadataTimestampSize = 8,
    MetadataSecretKeyOffset,
    MetadataSecretKeySize = 16,
    MetadataAutoIncrementOffset,
    MetadataAutoIncrementSize = 8,
    MetadataCountOffset,
    MetadataCountSize = 4,
    MetadataLastInternalIndexOffset,
    MetadataLastInternalIndexSize = 4,
    MetadataSize = 200,
    PageHeaderSize = 100,
    PageCellSize = 4,
    PageTypeOffset = 0,
    PageTypeSize = 4,
    PageIndexOffset = 4,
    PageIndexSize = 4,
    PageNextOffset = 8,
    PageNextSize = 4,
    PageCountOffset = 12,
    PageCountSize = 4,
    PageFreeOffset = 16,
    PageFreeSize = 4,
    RecordHeaderSize = 40,
    RecordHeaderIndexOffset = 0,
    RecordHeaderIndexSize = 4,
    RecordHeaderOrderOffset = 4,
    RecordHeaderOrderSize = 4,
    RecordHeaderLengthOffset = 8,
    RecordHeaderLengthSize = 4,
    RecordHeaderMaxLengthOffset = 12,
    RecordHeaderMaxLengthSize = 4,
    RecordHeaderDeletedOffset = 16,
    RecordHeaderDeletedSize = 1,
    RecordHeaderAliasIndexOffset = 17,
    RecordHeaderAliasIndexSize = 4,
    RecordHeaderAliasOrderOffset = 21,
    RecordHeaderAliasOrderSize = 4
}
export interface KlafServiceConstructorArguments {
    path: string;
    secretKey: Uint8Array;
    metadata: KlafMetadata;
    engine: DataEngine;
    journal?: DataJournal;
}
export declare enum KlafPageType {
    UnknownType = 0,
    InternalType = 1,
    OverflowType = 2,
    SystemReservedType = 3
}
export interface KlafPageHeader {
    type: KlafPageType;
    index: number;
    next: number;
    count: number;
    free: number;
}
export interface KlafRecord {
    rawRecord: number[];
    rawHeader: number[];
    rawPayload: number[];
    header: {
        id: string;
        aliasId: string;
        index: number;
        order: number;
        aliasIndex: number;
        aliasOrder: number;
        length: number;
        maxLength: number;
        deleted: number;
    };
    payload: string;
}
export interface KlafPickResult {
    page: KlafPageHeader;
    record: KlafRecord;
    order: number;
}
export declare class KlafService implements DataJournalContainer {
    static readonly ErrorBuilder: {
        new (): {};
        ERR_DB_ALREADY_EXISTS(file: string): Error;
        ERR_DB_INVALID(file: string): Error;
        ERR_DB_NO_EXISTS(file: string): Error;
        ERR_ALREADY_DELETED(recordId: string): Error;
        ERR_INVALID_RECORD(recordId: string): Error;
        ERR_UNSUPPORTED_ENGINE(): Error;
        ERR_DATABASE_CLOSING(): Error;
    };
    static readonly Bootloader: {
        new (): {
            isValidDatabase(engine: DataEngine): Promise<boolean>;
            existsDatabase(path: string, engine: DataEngine): Promise<boolean>;
            existsJournal(databasePath: string, journal: DataJournal): Promise<boolean>;
            parseMetadata(metadata: number[]): KlafMetadata;
            create(option: KlafCreateOption): Promise<KlafServiceConstructorArguments>;
            open(option: KlafCreateOption): Promise<KlafServiceConstructorArguments>;
        };
    };
    readonly path: string;
    readonly engine: DataEngine;
    readonly journal?: DataJournal;
    readonly pageHeaderSize: number;
    readonly pageSize: number;
    readonly maximumFreeSize: number;
    readonly payloadSize: number;
    readonly secretKey: Uint8Array;
    private readonly _locker;
    private _closing;
    private readonly _encodingIdCache;
    private readonly _decodingIdCache;
    private readonly _decodingRecordCache;
    private readonly _pageHeaderCache;
    private readonly _recordPositionCache;
    private readonly _recordCache;
    private readonly _metadata;
    constructor({ path, secretKey, metadata, engine, journal, }: KlafServiceConstructorArguments);
    get metadata(): KlafMetadata;
    get closing(): boolean;
    get locker(): Ryoiki;
    createIterable(length: number, fill: number): number[];
    private _createEncodingIdCache;
    private _createDecodingIdCache;
    private _createDecodingRecordCache;
    private _createPageHeaderCache;
    private _createRecordPositionCache;
    private _createRecordCache;
    createEmptyHeader({ type, index, next, count, free }?: Partial<KlafPageHeader>): number[];
    private createEmptyPayload;
    private createEmptyPage;
    addEmptyPage(header: Partial<KlafPageHeader>, incrementInternalIndex: boolean): Promise<number>;
    recordId(index: number, order: number): Promise<string>;
    get(index: number): Promise<number[]>;
    pagePosition(index: number): number;
    pagePayloadPosition(index: number): number;
    pageCellPosition(index: number, order: number): number;
    recordPosition(index: number, order: number): Promise<number>;
    parseRecordId(recordId: string): Promise<{
        index: number;
        order: number;
    }>;
    rawRecordId(recordId: string): Promise<number[]>;
    createRecord(id: string, data: number[]): Promise<number[]>;
    createCell(recordOffset: number): number[];
    getRecord(index: number, order: number): Promise<number[]>;
    parseRecord(record: number[]): Promise<KlafRecord>;
    getHeader(index: number): Promise<number[]>;
    parseHeader(header: number[]): KlafPageHeader;
    getHeadPageIndex(index: number): Promise<number>;
    internalPick(recordId: string, recursiveAlias: boolean): Promise<KlafPickResult>;
    getRecords(index: number): Promise<KlafRecord[]>;
    setPageHeader(header: KlafPageHeader): Promise<void>;
    setPagePayload(index: number, order: number, record: number[]): Promise<void>;
    setPage(header: KlafPageHeader, data: number[]): Promise<string>;
    internalPut(text: string | number[], autoIncrement: boolean): Promise<string>;
    setInternalRecord(index: number, order: number, record: number[]): Promise<number>;
    setOverflowedRecord(index: number, record: number[]): Promise<number>;
    isInternalRecord(record: number[]): Promise<boolean>;
    internalUpdate(id: string, text: string): Promise<{
        id: string;
        text: string;
    }>;
    internalDelete(recordId: string, countDecrement: boolean): Promise<void>;
    pick(recordId: string): Promise<KlafPickResult>;
    put(text: string): Promise<string>;
    update(recordId: string, text: string): Promise<string>;
    delete(recordId: string): Promise<void>;
    exists(recordId: string): Promise<boolean>;
    close(handler: any): Promise<void>;
    startBackup(handler: any): Promise<void>;
    endBackup(handler: any): Promise<void>;
    backup(pageIndex: number): Promise<void>;
    restoreJournal(handler: any): Promise<void>;
}
