import { BPTreeAsync, SerializeStrategyHead } from 'serializable-bptree';
import { Ryoiki } from 'ryoiki';
import { KlafMetadata, KlafService } from '../core/KlafService';
import { KlafComparator } from './KlafComparator';
import { type QueueNode } from './KlafStrategy';
import { Debounce } from '../utils/Debounce';
import { DataEngine } from '../engine/DataEngine';
import { DataJournal, DataJournalContainer } from '../engine/DataJournal';
import { KlafDocumentCreateOption } from './KlafDocument';
import { KlafRepositorySynchronizer } from './KlafRepositorySynchronizer';
export type PrimitiveType = string | number | boolean | null;
export type SupportedType = PrimitiveType | SupportedType[] | {
    [key: string]: SupportedType;
};
export interface KlafDocumentRoot {
    verify: 'TissueRollDocument';
    schemeVersion: number;
    reassignments: string[];
    head: Record<string, SerializeStrategyHead | null>;
}
export interface KlafDocumentMetadata {
    autoIncrement: bigint;
    count: number;
    payloadSize: number;
    timestamp: bigint;
    schemeVersion: number;
}
export interface KlafDocumentServiceConstructorArguments<S extends KlafDocumentable> {
    metadata: KlafMetadata;
    core: KlafService;
    root: KlafDocumentRoot;
    rootId: string;
    order: number;
    scheme: KlafDocumentScheme<S>;
    schemeVersion: number;
}
export type KlafDocumentQueryCondition<T, K extends keyof T = keyof T> = {
    /**
     * Includes if this value matches the document's property value.
     */
    equal?: T[K];
    /**
     * Includes if this value does not match the document's property value.
     */
    notEqual?: T[K];
    /**
     * Includes if this value is greater than the document's property value.
     */
    gt?: T[K];
    /**
     * Includes if this value is greater than or equal to the document's property value.
     */
    gte?: T[K];
    /**
     * Includes if this value is less than the document's property value.
     */
    lt?: T[K];
    /**
     * Includes if this value is less than or equal to the document's property value.
     */
    lte?: T[K];
    /**
     * Searches for documents matching the given pattern. '%' matches zero or more characters, and '_' matches exactly one character.
     */
    like?: string;
};
export interface KlafDocumentable {
    [key: string]: SupportedType;
}
export interface KlafDocumentBase {
    /**
     * The index when the document was inserted. This value is automatically added when inserted into the database.
     */
    documentIndex: number;
    /**
     * The timestamp when the document was created. This value is automatically added when inserted into the database.
     */
    createdAt: number;
    /**
     * The timestamp when the document was last updated. This value is automatically updated when the document is modified in the database.
     */
    updatedAt: number;
}
export type KlafDocumentShape<T extends KlafDocumentable> = T & KlafDocumentBase;
export type KlafDocumentQuery<T extends KlafDocumentShape<any>> = {
    [K in keyof T]?: T[K] | KlafDocumentQueryCondition<T, K>;
};
export interface KlafDocumentOption<T extends KlafDocumentShape<KlafDocumentable>> {
    /**
     * Used when retrieving a portion of the searched documents. Specifies the starting offset, with a default value of `0`.
     */
    start?: number;
    /**
     * Used when retrieving a portion of the searched documents. Specifies the ending offset, with a default value of `Number.MAX_SAFE_INTEGER`.
     */
    end?: number;
    /**
     * The property used for sorting the retrieved documents. Results are sorted based on this value, with the default being `documentIndex`.
     */
    order?: keyof KlafDocumentShape<T> & string;
    /**
     * The property used for sorting the retrieved documents. If set to `true`, it sorts in descending order. The default value is `false`.
     */
    desc?: boolean;
}
export interface KlafDocumentField<T extends SupportedType> {
    default: () => T;
    validate?: (v: SupportedType) => boolean;
}
export type KlafDocumentScheme<S extends KlafDocumentable> = {
    [K in keyof S]: KlafDocumentField<S[K]>;
};
export type KlafDocumentSchemeType<S extends KlafDocumentable, T extends KlafDocumentScheme<S>> = {
    [K in keyof T]: ReturnType<T[K]['default']>;
};
export declare class KlafDocumentService<S extends KlafDocumentable> implements DataJournalContainer {
    static readonly DB_NAME = "TissueRollDocument";
    static readonly ErrorBuilder: {
        new (): {};
        ERR_INVALID_OBJECT(stringify: string): Error;
        ERR_DATABASE_CLOSING(): Error;
        ERR_DB_ALREADY_EXISTS(file: string): Error;
        ERR_DB_INVALID(file: string): Error;
        ERR_DB_NO_EXISTS(file: string): Error;
        ERR_ALREADY_DELETED(recordId: string): Error;
        ERR_INVALID_RECORD(recordId: string): Error;
        ERR_UNSUPPORTED_ENGINE(): Error;
    };
    static readonly Bootloader: {
        new <S_1 extends KlafDocumentable, T extends KlafDocumentScheme<S_1>>(): {
            createOption({ version, core, }: {
                version: number;
                core: KlafService;
            }): {
                schemeVersion: number;
                metadata: KlafMetadata;
                order: number;
            };
            parsePayload(payload: string): object;
            isValidDatabase(object: object): boolean;
            existsDatabase(path: string, engine: DataEngine): Promise<boolean>;
            existsJournal(databasePath: string, journal: DataJournal): Promise<boolean>;
            create(option: KlafDocumentCreateOption<S_1, T>): Promise<KlafDocumentServiceConstructorArguments<S_1>>;
            open(option: KlafDocumentCreateOption<S_1, T>): Promise<KlafDocumentServiceConstructorArguments<S_1>>;
        };
    };
    static OrderN(payloadSize: number, meanValueSize: number): number;
    readonly core: KlafService;
    readonly journal?: DataJournal;
    readonly rootId: string;
    readonly order: number;
    readonly comparator: KlafComparator;
    readonly synchronizer: KlafRepositorySynchronizer<QueueNode>;
    readonly scheme: KlafDocumentScheme<S>;
    readonly locker: Ryoiki;
    readonly debounce: Debounce;
    readonly schemeVersion: number;
    private _createdTrees;
    private _closing;
    private readonly _trees;
    private readonly _document;
    private readonly _treeDeleteQueue;
    private readonly _treeUpdateQueue;
    private readonly _treeTempNodes;
    private _root;
    private _metadata;
    constructor({ metadata, core, root, rootId, order, scheme, schemeVersion, }: KlafDocumentServiceConstructorArguments<S>);
    alterScheme(schemeVersion: number): Promise<void>;
    createTrees(): Promise<void>;
    private _createDocumentCache;
    protected getTree(property: string): BPTreeAsync<string, SupportedType> | null;
    protected updateRoot(root: KlafDocumentRoot): Promise<void>;
    private _normalizeOption;
    normalizeFlatQuery(query: KlafDocumentQuery<KlafDocumentShape<S>>): KlafDocumentQuery<KlafDocumentShape<S>>;
    normalizeQuery(query: KlafDocumentQuery<KlafDocumentShape<S>>, properties: Set<keyof KlafDocumentShape<S>>): KlafDocumentQuery<KlafDocumentShape<S>>;
    normalizeRecord(record: Partial<S>): S;
    get metadata(): KlafDocumentMetadata;
    get closing(): boolean;
    get engine(): DataEngine;
    callInternalPut(document: Partial<S>, ...overwrite: Partial<S>[]): Promise<KlafDocumentShape<S>>;
    put(document: Partial<S>): Promise<KlafDocumentShape<S>>;
    delete(query: KlafDocumentQuery<KlafDocumentShape<S>>): Promise<number>;
    callInternalUpdate(query: KlafDocumentQuery<KlafDocumentShape<S>>, update: Partial<S | KlafDocumentShape<S>> | ((record: KlafDocumentShape<S>) => Partial<S>), createOverwrite: (before: KlafDocumentShape<S>) => Partial<KlafDocumentShape<S>>): Promise<number>;
    partialUpdate(query: KlafDocumentQuery<KlafDocumentShape<S>>, update: Partial<S> | ((record: KlafDocumentShape<S>) => Partial<S>)): Promise<number>;
    fullUpdate(query: KlafDocumentQuery<KlafDocumentShape<S>>, update: S | ((record: KlafDocumentShape<S>) => S)): Promise<number>;
    protected findRecordIds(query: KlafDocumentQuery<KlafDocumentShape<S>>, order?: keyof KlafDocumentShape<S> & string, desc?: boolean): Promise<string[]>;
    pick(query: KlafDocumentQuery<KlafDocumentShape<S>>, option?: KlafDocumentOption<KlafDocumentShape<S>>): Promise<KlafDocumentShape<S>[]>;
    count(query: KlafDocumentQuery<KlafDocumentShape<S>>): Promise<number>;
    close(): Promise<void>;
}
