import { type DataEngine } from './DataEngine';
/**
 * Represents a container for data journal operations.
 * It provides the interface for interacting with the data journal.
 */
export interface DataJournalContainer {
    /**
     * The `DataJournal` instance associated with this container.
     * It handles backup and restore operations for the database.
     */
    journal?: DataJournal;
}
export declare enum DataJournalFormat {
    /**
     * The size of the root header in the journal file.
     */
    RootHeaderSize = 100,
    /**
     * The offset for the working flag in the root header.
     */
    RootWorkingOffset = 0,
    /**
     * The size of the working flag.
     */
    RootWorkingSize = 1,
    /**
     * The offset for the maximum page index in the root header.
     */
    RootMaxPageIndexOffset = 1,
    /**
     * The size of the maximum page index.
     */
    RootMaxPageIndexSize = 4,
    /**
     * The offset for the database metadata in the journal file.
     */
    DBMetadataOffset = 100,
    /**
     * The size of the database metadata.
     */
    DBMetadataSize = 200,
    /**
     * The offset for the database page in the journal file.
     */
    DBPageOffset = 300,
    /**
     * The suffix used for journal files.
     */
    JournalSuffix = ".journal"
}
export interface DataJournalInitialOption {
    working: number;
    maximumPageIndex: number;
}
type DataJournalMetadataFormatKey = keyof DataJournalInitialOption;
type DataJournalMetadataFormatValue = [number, number];
type DataJournalMetadataFormatMap = Record<DataJournalMetadataFormatKey, DataJournalMetadataFormatValue>;
interface DataJournalMetadataFormData {
    dirty: boolean;
    numeric: number | bigint;
    array: number[];
}
type DataJournalMetadataFormatState = Record<DataJournalMetadataFormatKey, DataJournalMetadataFormData>;
declare class DataJournalInitialOptionManager {
    readonly engine: DataEngine;
    readonly formatMap: DataJournalMetadataFormatMap;
    readonly state: Partial<DataJournalMetadataFormatState>;
    constructor(engine: DataEngine, formatMap: DataJournalMetadataFormatMap);
    private _read;
    read(key: DataJournalMetadataFormatKey, to: 'numeric'): Promise<number | bigint>;
    read(key: DataJournalMetadataFormatKey, to: 'array'): Promise<number[]>;
    write(key: DataJournalMetadataFormatKey, value: number): void;
    write(key: DataJournalMetadataFormatKey, value: number[]): void;
    commit(): Promise<void>;
}
/**
 * The `DataJournal` class manages a journal file for backing up and restoring a database.
 * It tracks changes made to the database and provides mechanisms for rolling back to a previous state
 * in case of errors or data corruption.
 */
export declare class DataJournal {
    /**
     * The initial data used when creating a new journal file.
     */
    protected static readonly InitialData: number[];
    /**
     * Decorator class for `DataJournal` methods.
     * It provides decorators for common checks, like initialization and ownership.
     */
    static readonly Decorator: {
        new (): {};
        /**
         * Verifies that the journal has been initialized.
         * @param target The target object.
         * @param property The property key.
         * @param descriptor The property descriptor.
         * @returns The modified property descriptor.
         * @throws {Error} If the journal is not initialized.
         */
        VerifyInitialized(target: DataJournalContainer, property: PropertyKey, descriptor: PropertyDescriptor): PropertyDescriptor;
        /**
         * Requires the journal to be initialized before allowing the decorated method to execute.
         * If the journal is not initialized, the method will not be executed.
         * @param target The target object.
         * @param property The property key.
         * @param descriptor The property descriptor.
         * @returns The modified property descriptor.
         */
        RequireInitialized(target: DataJournalContainer, property: PropertyKey, descriptor: PropertyDescriptor): PropertyDescriptor;
        /**
         * Requires the `DataJournalContainer` to have `DataJournal` instance before allowing the decorated method to execute.
         * If there is no `DataJournalToken` instance, the method will not be executed.
         * @param target The target object.
         * @param property The property key.
         * @param descriptor The property descriptor.
         * @returns The modified property descriptor.
         */
        RequireInstance(target: DataJournalContainer, property: PropertyKey, descriptor: PropertyDescriptor): PropertyDescriptor;
        /**
         * Requires the `DataJournalContainer` to have `DataJournal` instance and handler before allowing the decorated method to execute.
         * If there is no `DataJournalToken` instance or handler, the method will not be executed.
         * @param target The target object.
         * @param property The property key.
         * @param descriptor The property descriptor.
         * @returns The modified property descriptor.
         */
        RequireAccessibleHandler(target: DataJournalContainer, property: PropertyKey, descriptor: PropertyDescriptor): PropertyDescriptor;
    };
    /**
     * Creates an error for when the journal needs to be initialized.
     * @returns An error object.
     */
    protected static ERR_REQUIRE_INIT(): Error;
    /**
     * Creates an error for when the journal requires a token.
     * @returns An error object.
     */
    protected static ERR_REQUIRE_TOKEN(): Error;
    /**
     * Creates an error for when the journal is already initialized.
     * @returns An error object.
     */
    protected static ERR_ALREADY_INIT(): Error;
    /**
     * Creates an error for when the journal requires proper permissions.
     * @param handler The handler trying to access.
     * @param accessibleHandler The handler that has access.
     * @returns An error object.
     */
    protected static ERR_REQUIRE_PERM(handler: any, accessibleHandler: any): Error;
    /**
     * Creates an iterable of a specific length and filled with a specific value.
     * @param length The length of the iterable.
     * @param fill The value to fill the iterable with.
     * @returns A number array.
     */
    protected static CreateIterable(length: number, fill: number): number[];
    /**
     * Checks if an object is an instance of `DataJournal`.
     * @param object The object to check.
     * @returns True if the object is a `DataJournal` instance, false otherwise.
     */
    static IsInstance(object: any): object is DataJournal;
    private readonly _indexes;
    protected initialized: boolean;
    protected transacting: boolean;
    protected handler: any;
    protected readonly initialOption: DataJournalInitialOptionManager;
    readonly engine: DataEngine;
    /**
     * Creates a new `DataJournal` instance.
     * @param engine The data engine to use for journal operations.
     */
    constructor(engine: DataEngine);
    /**
     * Gets whether the journal is initialized.
     * @returns True if initialized, false otherwise.
     */
    get isInitialized(): boolean;
    /**
     * Gets whether a transaction is in progress.
     * @returns True if transacting, false otherwise.
     */
    get isTransacting(): boolean;
    /**
     * Gets the journal file path for a given database path.
     * @param databasePath The path to the database file.
     * @returns The path to the journal file.
     */
    getJournalPath(databasePath: string): string;
    /**
     * Checks if the given handler has access to the journal.
     * @param handler The handler to check.
     * @returns True if the handler has access, false otherwise.
     */
    isAccessible(handler: any): boolean;
    isAlreadyBackup(pageIndex: number): boolean;
    /**
     * Checks if a journal file exists for a given database path.
     * @param databasePath The path to the database file.
     * @returns True if the journal file exists, false otherwise.
     */
    exists(databasePath: string): Promise<boolean>;
    /**
     * Creates or updates a journal file with the given path and header data.
     * @param databasePath The path to the database file.
     * @param headerData The header data to write to the journal file.
     */
    make(databasePath: string, headerData: number[]): Promise<void>;
    /**
     * Resets the journal with the given header data.
     * @param handler The handler performing the reset.
     * @param metadata The header data to use for the reset.
     * @throws {Error} If the journal is not initialized or the handler does not have permission.
     */
    reset(handler: any, metadata: number[]): Promise<void>;
    /**
     * Starts a transaction in the journal.
     * @param handler The handler starting the transaction.
     * @throws {Error} If the journal is not initialized, the handler does not have permission, or a transaction is already in progress.
     */
    startTransaction(handler: any, initialOption: Partial<DataJournalInitialOption>): Promise<void>;
    /**
     * Ends a transaction in the journal.
     * @param handler The handler ending the transaction.
     * @throws {Error} If the journal is not initialized, the handler does not have permission, or no transaction is in progress.
     */
    endTransaction(handler: any, initialOption: Partial<DataJournalInitialOption>): Promise<void>;
    close(handler: any, databasePath: string): Promise<void>;
    /**
     * Restores the database state from the journal.
     * @param handler The handler performing the restoration.
     * @param option.pageSize The size of each page in the database.
     * @param option.getPageIndex A function to extract the page index from page data.
     * @param option.restoreMetadata A function to restore the database metadata.
     * @param option.restorePage A function to restore a specific page of the database.
     * @param option.truncate A function to truncate the database to a specific size.
     * @returns A promise that resolves when the restoration is complete.
     * @throws {Error} If the journal is not initialized or the handler does not have permission.
     */
    restore(handler: any, { pageSize, getPageIndex, restoreMetadata, restorePage, truncate, }: {
        pageSize: number;
        getPageIndex: (pageData: number[]) => number;
        restoreMetadata: (metadata: number[]) => Promise<void>;
        restorePage: (pageIndex: number, pageData: number[]) => Promise<void>;
        truncate: (maximumPageIndex: number) => Promise<void>;
    }): Promise<void>;
    /**
     * Backs up a page of the database to the journal.
     * @param pageIndex The index of the page to back up.
     * @param data The data of the page.
     */
    backupPage(pageIndex: number, data: number[]): Promise<void>;
    /**
     * Gets the backed-up metadata from the journal.
     * @returns The metadata.
     */
    getMetadataBackup(): Promise<number[]>;
    /**
     * Asynchronously yields backed-up pages from the journal.
     * @param pageSize The size of each page.
     * @returns An asynchronous generator that yields backed-up pages.
     */
    getBackupPages(pageSize: number): AsyncGenerator<number[]>;
}
/**
 * `AuthenticatedDataJournal` class
 * It is for providing authentication of `DataJournal` class.
 */
export declare class AuthenticatedDataJournal<T> extends DataJournal {
    /**
     * Checks if an object is an instance of `AuthenticatedDataJournal`.
     * @param object The object to check.
     * @returns True if the object is a `AuthenticatedDataJournal` instance, false otherwise.
     */
    static IsInstance<T = any>(object: any): object is AuthenticatedDataJournal<T>;
    /**
     * Creates an `AuthenticatedDataJournal` instance from a `DataJournal` instance.
     * @param guessHandler The handler for authentication.
     * @param journal The `DataJournal` instance.
     * @returns An `AuthenticatedDataJournal` instance or `undefined` if the journal is not provided.
     */
    static From(guessHandler: any, journal: DataJournal | undefined): AuthenticatedDataJournal<any> | undefined;
    /**
     * Creates a new `AuthenticatedDataJournal` instance.
     * @param handler The handler for authentication.
     * @param engine The data engine to use for journal operations.
     */
    constructor(handler: T, engine: DataEngine);
}
export {};
